var sqlite3=require('sqlite3')
var db = new sqlite3.Database('data/data.db');
var crypto = require("crypto");
db.serialize()
db.exec(`
	CREATE TABLE IF NOT EXISTS "urls" (
		"id"	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
		"browserId"	TEXT NOT NULL,
		"url"	TEXT NOT NULL
		
	);
	CREATE TABLE IF NOT EXISTS "filters" (
		"filterId" INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
		"urlId" INTEGER NOT NULL,
		FOREIGN KEY(urlId) REFERENCES urls(id)
	);
	CREATE TABLE IF NOT EXISTS "filtersActions" (
		"id"	INTEGER NOT NULL PRIMARY KEY AUTOINCREMENT,
		"filterId"	INTEGER NOT NULL,
		"field"	TEXT NOT NULL,
		"text" TEXT NOT NULL,
		"action" TEXT NOT NULL,
		"description"	TEXT,
		FOREIGN KEY(filterId) REFERENCES filters(filterId)
	);
	CREATE TABLE IF NOT EXISTS "replace" (
		"id"	INTEGER NOT NULL,
		"field" TEXT,
		"from"	TEXT NOT NULL,
		"to"	TEXT,
		"description"	TEXT,
		FOREIGN KEY(id) REFERENCES urls(id)
	);
	CREATE TABLE IF NOT EXISTS "regexReplace" (
		"id"	INTEGER NOT NULL,
		"field" TEXT,
		"from"	TEXT NOT NULL,
		"to"	TEXT,
		"description"	TEXT,
		FOREIGN KEY(id) REFERENCES urls(id)
	);
`,function(err){console.log(err)})
function createUrl(url,callback) {
	var id=crypto.randomBytes(20).toString('hex');
	db.run("INSERT INTO urls('browserId','url') VALUES ((?),(?));",[id, url],function (err) {
		if(err) console.log('error inserting url:',err)
		callback(id)
	})
}
function addFilter(id,filter) {
	db.run("INSERT INTO filters('urlId') VALUES ((?));",[id],function(err) {
		if(err) console.log('error inserting filter:',err)
		var filterId=this.lastID
		filter.forEach(function(action) {
			db.run('INSERT INTO filtersActions(filterId,field,action,text,description) VALUES (?,?,?,?,?)',[filterId,action.field,action.action,action.text,action.description])
		})
	})
}
function addReplace(id,field,from,to,table,description) {
	db.run("INSERT INTO "+table+"('id','field','from','to','description') VALUES ((?),(?),(?),(?),(?));",[id,field,from,to,description],function(err) {if(err) console.log('error inserting replace:',err)})
}
function removeAllFromId(id,callback) {
	db.each('SELECT filterId FROM filters WHERE urlId = (?)',[id],function (err,row) {
		db.run('DELETE FROM filtersActions WHERE filterId = ?',[row.filterId],function () {})
	}, function () {
		console.log('removing filter!')
		db.run('DELETE FROM filters WHERE urlId = ?',[id],function (err) {callback()})
	})
	db.run('DELETE FROM replace WHERE id = ?',[id],function (err) {})
	db.run('DELETE FROM regexReplace WHERE id = ?',[id],function (err) {})
}
function getUrl(id) {
	return new Promise(function(resolve,reject) {
		db.get('SELECT url FROM urls WHERE id = ?',[id],function(err,row) {
			if(row) resolve(row.url)
		})
	})
}
function getFilters(id) {
	return new Promise(function(resolve,reject) {
		var filters={}
		db.each('SELECT filters.filterId AS filterId,field,text,action,description FROM filtersActions INNER JOIN filters ON filters.filterId = filtersActions.filterId WHERE filters.urlId=? ORDER BY id',[id],function(err,row) {
			if(!filters[row.filterId]) filters[row.filterId]=[]
			filters[row.filterId].push({field:row.field,action:row.action,text:row.text,description:row.description})
		},function () {resolve(Object.values(filters))})
	})
}
function getReplace(id,table) {
	return new Promise(function(resolve,reject) {
		db.all('SELECT "field","from","to","description" FROM '+table+' WHERE id = ?',[id],function(err,rows) {
			if(!rows) rows=[]
			resolve(rows)
		})
	})
}
function updateUrl(id,url) {
	db.run('UPDATE urls SET url = ? WHERE id = ?',[url,id],function (err) {})
}
function updateAll(id,url,filters,replace,regexReplace) {
	getDatabaseId(id,function(id) {
		removeAllFromId(id,function () {
			filters.forEach(function(filter) {addFilter(id,filter)})
			replace.forEach(function(replace) {addReplace(id,replace.field,replace.from,replace.to,'replace',replace.description)})
			regexReplace.forEach(function(replace) {addReplace(id,replace.field,replace.from,replace.to,'regexReplace',replace.description)})
		})
		updateUrl(id,url)
	})
}
function getDatabaseId(browserId,callback) {
	db.get('SELECT id FROM urls WHERE browserId = ?',[browserId],function(err,row) {
		if(row) callback(row.id)
	})
}
function getSettings(browserId,callback) {
	getDatabaseId(browserId,function(id) {
		Promise.all([getUrl(id),getFilters(id),getReplace(id,'replace'),getReplace(id,'regexReplace')]).then(function(result) {
			callback({url:result[0],filters:result[1],replace:result[2],regexReplace:result[3],browserId})
		})
	})
}
module.exports={createUrl:createUrl,updateAll:updateAll,getSettings:getSettings}
