process.chdir(__dirname);
var http=require('http')
var https=require('https')
var fs=require('fs')
var dbfuncs=require('./database.js')
var globalFuncs=require('./global.js')
function toObject(eventText) {
	event={}
	globalFuncs.replaceAll(eventText,'\n ','').split('\n').forEach(function(eventSplit) {
		parts=eventSplit.split(':')
		event[parts.shift().split(';')[0]]=parts.join(':')
	})
	return event
}
function fromObject(event) {
	return Object.keys(event).map(function(key) {return key+':'+event[key]}).join('\n')
}
function doRequest(url,callback) {
	var protocol=url.split(':')[0]
	var module
	if(protocol=='http') module=http
	else if(protocol=='https') module=https
	console.log('no http or https found!')
	if(!module) return
	module.get(url,{headers: {'User-Agent': 'rooster script'}},function(res) {
		if(res.headers.location) {
			console.log('following redirect to',res.headers.location)
			doRequest(res.headers.location,callback)
			return
		}
		var data=''
		res.on('data',function(chunk) {data+=chunk})
		res.on('end',function() {
			data=globalFuncs.replaceAll(data,"\r",'')
			if(data.split('\n')[0]=="BEGIN:VCALENDAR") callback(data)
		})
	}).on('error',function () {})
}
function totalToArray(data, browserId) {
	var events=data.split('BEGIN:VEVENT\n')
	events.shift()
	events=events.map(function (event) {return event.split('\nEND:VEVENT')[0]})
	events=events.map(toObject)
	date=new Date(new Date()+'UTC').toISOString().replace(/[:-]/g,'').split('.')[0]
	events.push({'DTSTART;TZID=Europe/Amsterdam':date,'DTEND;TZID=Europe/Amsterdam':date,SUMMARY:'Laatst gesynchroniseerd',UID:'Ruben-rooster',DESCRIPTION:'Laatst gesynchroniseerd (lokale tijd:'+date+')\nDe instellingen zijn te vinden op https://rooster.rubend.nl/settings/#'+browserId})
	return events
}
function fromTotalArray(data) {
	var eventTexts=data.map(fromObject)
	response=eventTexts.map(function(event) {
		event+='\nBEGIN:VALARM\nDESCRIPTION:reminder\nACTION:DISPLAY\nTRIGGER:-PT30M\nX-KDE-KCALCORE-ENABLED:TRUE\nEND:VALARM'
		return 'BEGIN:VEVENT\n'+event+'\nEND:VEVENT'
	}).join('\n')
	return response
}
function generateUpdatedObject(settings,data) {
	events=totalToArray(data,settings.browserId)
	accepted=events.filter(function(text) {return globalFuncs.filterEvent(text,settings.filters)})
	replaced=accepted.map(function(object) {return globalFuncs.replaceEvent(object,settings.regexReplace,false,true)})
	replaced=replaced.map(function(object) {return globalFuncs.replaceEvent(object,settings.replace)})
	return replaced
}
function generateUpdatedIcal(settings,data) {
	var response=data.split('BEGIN:VEVENT\n').shift()
	var updated=generateUpdatedObject(settings,data)
	response+=fromTotalArray(updated)
	response+='\nEND:VCALENDAR'
	return response
}
server=http.createServer(function(req,res) {
	data=''
	req.on('data',function(chunk) {
		data+=chunk
	})
	req.on('end',function () {
		if(req.method=="PUT") return
		if(!req.url.includes('/api')) {
			path='rooster'+req.url.split('#')[0]
			try {
				res.end(fs.readFileSync(path,'utf8'))
			} catch(e) {
				console.log(e)
				try {
					res.writeHead(200,{'Content-Type': 'text/html'})
					res.end(fs.readFileSync(path+'/index.html','utf8'))
				} catch (e) {
					console.log(e)
				}
			}
			return
		}
		query=req.url.split('/').pop().split('?')
		if(data) {
			try {
				data=JSON.parse(data)
			} catch (e) {
				console.log('error parsing body:', e)
				return
			}
			if(!data.id) {
				if(data.url) dbfuncs.createUrl(data.url,function (response) {
					res.writeHead(200,{'Content-Type': 'application/json','Access-Control-Allow-Origin':'*'})
					res.end(JSON.stringify(response))
				})
				else {
					console.log('didn\'t receive url!')
					return
				}
			}
			dbfuncs.getSettings(data.id,function(settings) {
				dbfuncs.updateAll(data.id,data.url,data.filters,data.replace,data.regexReplace)
				res.writeHead(200,{'Content-Type': 'application/json','Access-Control-Allow-Origin':'*'})
				setTimeout(function () {
					res.end(JSON.stringify({}))
				},2000)
			})
		}
		var id=query[0]
		var action=query[1]
		const browserId=id;
		console.log(id,action)
		dbfuncs.getSettings(id,function(settings) {
			if(action=="settings") {
				console.log('requested settings')
				res.writeHead(200,{'Content-Type': 'application/json','Access-Control-Allow-Origin':'*'})
				res.end(JSON.stringify(settings))
				return
			} else {
				console.log('not requested settings')
				doRequest(settings.url,function(data) {
					if(action=="ical") {
						res.writeHead(200,{'Access-Control-Allow-Origin':'*', 'Content-Type':'text/plain; charset=utf-8'})
						res.end(generateUpdatedIcal(settings,data))
					}
					if(action=="json") {
						res.writeHead(200,{'Content-Type': 'application/json','Access-Control-Allow-Origin':'*'})
						res.end(JSON.stringify(totalToArray(data,browserId)))
					}
				})
			}
		})
	})
})
server.listen(8080)
exports.stop=function () {
	server.close()
}
exports.listening=function () {
	return server.listening
}
